/*
 * Author: Dušan Kolář
 */

#ifndef PARSE_COAP
#define PARSE_COAP


typedef enum e_parse_error {
    PE_OK,
    PE_TOO_SHORT,                 // passed buffer is shorter then smallest packet
    PE_OUT_OF_DATA,               // running out of the passed data length
    PE_WRONG_PAYLOAD_MARKER,      // payload marker is either 0xfN or 0xNf, where N<0xf
    PE_EXPECTED_VERSION_01,       // version not 01
    PE_TOKEN_LENGTH_SIZE,         // token length >8
    PE_ZERO_LENGTH_PAYLOAD,       // payload marker present, but zero length payload present
    PE_OUT_OF_MEMORY,             // malloc failed to return free memory thunk
    PE_OPTION_OUT_OF_BOUNDS,      // option length is out of allowed limits
    PE_UINT_GT64,                 // uint option is larger than 64 bits
} parse_error_t;


#include <inttypes.h>


typedef struct s_str_opt {
    uint16_t     number;          // option number
    uint16_t     length;          // length of string
    char        *string;          // string option data
} str_opt_t;

typedef struct s_opaq_opt {
    uint16_t     number;          // option number
    uint16_t     length;          // length of opaque data
    uint8_t     *opaque;          // opaque option data
} opaq_opt_t;

typedef struct s_uint_opt {
    uint16_t     number;          // option number
    uint64_t     uint_val;        // option of type uint (hopefully not larger int will be used
} uint_opt_t;

typedef struct s_coap_data {
    unsigned int type;            // message type
    uint8_t      code;            // message code
    unsigned int tkl;             // token length
    uint8_t      token[8];        // token contents
    uint16_t     id;              // message id
    uint16_t     options_num;     // number of options in total
    uint16_t     str_num;         // number of string options
    str_opt_t  **strings;         // (pointer to) string options array
    uint16_t     opaq_num;        // number of opaque options
    opaq_opt_t **opaques;         // (pointer to) opaque options array
    uint16_t     uint_num;        // number of opaque options
    uint_opt_t **uints;           // (pointer to) uint options array
    uint16_t     empty_num;       // number of empty options
    uint16_t    *emptys;          // array of empty option numbers
    uint16_t     payload_length;  // payload length
    uint8_t     *payload;         // payload itself
} coap_data_t;


coap_data_t *parse_coap(uint8_t *input, uint32_t size, parse_error_t *error);
void         free_coap_data(coap_data_t *data);

#endif
